Attribute VB_Name = "modMath"
Option Explicit

Public Type Vector
  x As Single
  y As Single
  z As Single
End Type

Public Type Matrix
  m11 As Single: m12 As Single: m13 As Single
  m21 As Single: m22 As Single: m23 As Single
  m31 As Single: m32 As Single: m33 As Single
  m41 As Single: m42 As Single: m43 As Single
End Type

Public Const Pi As Single = 3.141593

Public Sub MatrixIdentity(MOut As Matrix)
  MOut.m11 = 1:     MOut.m12 = 0:     MOut.m13 = 0
  MOut.m21 = 0:     MOut.m22 = 1:     MOut.m23 = 0
  MOut.m31 = 0:     MOut.m32 = 0:     MOut.m33 = 1
  MOut.m41 = 0:     MOut.m42 = 0:     MOut.m43 = 0
End Sub

Public Sub MatrixRotationX(MOut As Matrix, ByVal a As Double)
  Dim CosA As Single, SinA As Single
  CosA = Cos(a)
  SinA = Sin(a)
  MOut.m11 = 1:     MOut.m12 = 0:     MOut.m13 = 0
  MOut.m21 = 0:     MOut.m22 = CosA:  MOut.m23 = SinA
  MOut.m31 = 0:     MOut.m32 = -SinA: MOut.m33 = CosA
  MOut.m41 = 0:     MOut.m42 = 0:     MOut.m43 = 0
End Sub

Public Sub MatrixRotationY(MOut As Matrix, ByVal a As Double)
  Dim CosA As Single, SinA As Single
  CosA = Cos(a)
  SinA = Sin(a)
  MOut.m11 = CosA:  MOut.m12 = 0:     MOut.m13 = -SinA
  MOut.m21 = 0:     MOut.m22 = 1:     MOut.m23 = 0:
  MOut.m31 = SinA:  MOut.m32 = 0:     MOut.m33 = CosA
  MOut.m41 = 0:     MOut.m42 = 0:     MOut.m43 = 0
End Sub

Public Sub MatrixRotationZ(MOut As Matrix, ByVal a As Double)
  Dim CosA As Single, SinA As Single
  CosA = Cos(a)
  SinA = Sin(a)
  MOut.m11 = CosA:  MOut.m12 = -SinA: MOut.m13 = 0
  MOut.m21 = SinA:  MOut.m22 = CosA:  MOut.m23 = 0
  MOut.m31 = 0:     MOut.m32 = 0:     MOut.m33 = 1
  MOut.m41 = 0:     MOut.m42 = 0:     MOut.m43 = 0
End Sub

Public Sub MatrixTranslation(MOut As Matrix, ByVal x As Single, ByVal y As Single, ByVal z As Single)
  MOut.m11 = 1:     MOut.m12 = 0:     MOut.m13 = 0
  MOut.m21 = 0:     MOut.m22 = 1:     MOut.m23 = 0
  MOut.m31 = 0:     MOut.m32 = 0:     MOut.m33 = 1
  MOut.m41 = x:     MOut.m42 = y:     MOut.m43 = z
End Sub

Public Sub MatrixScaling(MOut As Matrix, ByVal x As Single, ByVal y As Single, ByVal z As Single)
  MOut.m11 = x:     MOut.m12 = 0:     MOut.m13 = 0
  MOut.m21 = 0:     MOut.m22 = y:     MOut.m23 = 0
  MOut.m31 = 0:     MOut.m32 = 0:     MOut.m33 = z
  MOut.m41 = 0:     MOut.m42 = 0:     MOut.m43 = 0
End Sub

Public Sub MatrixMultiply(MOut As Matrix, M1 As Matrix, M2 As Matrix)
  Dim M As Matrix

  M.m11 = M1.m11 * M2.m11 + M1.m12 * M2.m21 + M1.m13 * M2.m31
  M.m12 = M1.m11 * M2.m12 + M1.m12 * M2.m22 + M1.m13 * M2.m32
  M.m13 = M1.m11 * M2.m13 + M1.m12 * M2.m23 + M1.m13 * M2.m33

  M.m21 = M1.m21 * M2.m11 + M1.m22 * M2.m21 + M1.m23 * M2.m31
  M.m22 = M1.m21 * M2.m12 + M1.m22 * M2.m22 + M1.m23 * M2.m32
  M.m23 = M1.m21 * M2.m13 + M1.m22 * M2.m23 + M1.m23 * M2.m33

  M.m31 = M1.m31 * M2.m11 + M1.m32 * M2.m21 + M1.m33 * M2.m31
  M.m32 = M1.m31 * M2.m12 + M1.m32 * M2.m22 + M1.m33 * M2.m32
  M.m33 = M1.m31 * M2.m13 + M1.m32 * M2.m23 + M1.m33 * M2.m33

  M.m41 = M1.m41 * M2.m11 + M1.m42 * M2.m21 + M1.m43 * M2.m31 + M2.m41
  M.m42 = M1.m41 * M2.m12 + M1.m42 * M2.m22 + M1.m43 * M2.m32 + M2.m42
  M.m43 = M1.m41 * M2.m13 + M1.m42 * M2.m23 + M1.m43 * M2.m33 + M2.m43

  MOut = M
End Sub

Public Sub VectorSubtract(vOut As Vector, v1 As Vector, v2 As Vector)
  vOut.x = v1.x - v2.x
  vOut.y = v1.y - v2.y
  vOut.z = v1.z - v2.z
End Sub

Public Sub VectorAdd(vOut As Vector, v1 As Vector, v2 As Vector)
  vOut.x = v1.x + v2.x
  vOut.y = v1.y + v2.y
  vOut.z = v1.z + v2.z
End Sub

Public Sub VectorNormalize(vOut As Vector, v As Vector)
  Dim iL As Single
  iL = 1 / Sqr(v.x * v.x + v.y * v.y + v.z * v.z)
  vOut.x = v.x * iL
  vOut.y = v.y * iL
  vOut.z = v.z * iL
End Sub

Public Sub VectorScale(vOut As Vector, v As Vector, s As Single)
  vOut.x = v.x * s
  vOut.y = v.y * s
  vOut.z = v.z * s
End Sub

Public Sub VectorCross(vOut As Vector, v1 As Vector, v2 As Vector)
  Dim x As Single, y As Single, z As Single
  x = v1.y * v2.z - v1.z * v2.y
  y = v1.z * v2.x - v1.x * v2.z
  z = v1.x * v2.y - v1.y * v2.x
  vOut.x = x: vOut.y = y: vOut.z = z
End Sub

Public Function VectorLen(v As Vector) As Single
  VectorLen = Sqr(v.x * v.x + v.y * v.y + v.z * v.z)
End Function

Public Function VectorDot(v1 As Vector, v2 As Vector) As Single
  VectorDot = v1.x * v2.x + v1.y * v2.y + v1.z * v2.z
End Function

Public Function Vector(ByVal x As Single, ByVal y As Single, ByVal z As Single) As Vector
  Vector.x = x
  Vector.y = y
  Vector.z = z
End Function
