Attribute VB_Name = "modSoft3D"
Option Explicit

Public Const MinDWZ As Single = 0.00001

Private Type BITMAPINFOHEADER
  biSize As Long
  biWidth As Long
  biHeight As Long
  biPlanes As Integer
  biBitCount As Integer
  biCompression As Long
  biSizeImage As Long
  biXPelsPerMeter As Double
  biClrUsed As Double
End Type

Private Type BITMAPINFO
  bmiHeader As BITMAPINFOHEADER
  bmiColors As Long
End Type

Public Type vData
  x As Single
  y As Single
  z As Single
  n As Vector
  u As Single
  v As Single
End Type

Public Type tData
  x As Long
  y As Long
  z As Single
  w As Single
  u As Single
  v As Single
  Lz As Single
  Lw As Single
  Lu As Single
  Lv As Single
  cD As Single
  cS As Single
End Type

Public Type pData
  x As Single
  z As Single
  w As Single
  u As Single
  v As Single
  Lu As Single
  Lv As Single
  Lz As Single
  Lw As Single
  cD As Single
  cS As Single
End Type

Public Type Color
  b As Single
  g As Single
  r As Single
  a As Single
End Type

Public Enum CullMode
  CullNone = 0
  CullCW = 1
  CullCCW = 2
End Enum

Public Enum DrawMode
  DrawTex = 0
  DrawATex = 1
  DrawFTex = 2
  DrawLTex = 4
  DrawAFTex = DrawATex Or DrawFTex
  DrawALTex = DrawATex Or DrawLTex
  DrawFLTex = DrawFTex Or DrawLTex
  DrawAFLTex = DrawATex Or DrawFTex Or DrawLTex
  DrawToZ = &H10000
End Enum

Private Declare Function SetDIBitsToDevice Lib "gdi32" (ByVal hDC As Long, ByVal x As Long, ByVal y As Long, ByVal dx As Long, ByVal dy As Long, ByVal SrcX As Long, ByVal SrcY As Long, ByVal Scan As Long, ByVal NumScans As Long, Bits As Any, BitsInfo As BITMAPINFO, ByVal wUsage As Long) As Long

Dim bi32BitInfo As BITMAPINFO, SR_hDC As Long
Dim lHalo1 As Single, lHalo2 As Single

Public Cull As CullMode
Public Draw As DrawMode
Public kXMul As Single, kXAdd As Single, kYMul As Single, kYAdd As Single
Public SR_kXMul As Single, SR_kXAdd As Single, SR_kYMul As Single, SR_kYAdd As Single
Public SR_Width As Long, SR_Height As Long
Public RTWidth As Long, RTHeight As Long
Public CBuffer() As Long, WBuffer() As Single, ZBuffer() As Single
Public Diffuse As Color, Ambient As Color, Specular As Color, vLite As Vector, lBrite As Single
Public MView As Matrix, vCam As Vector
Public MViewLight As Matrix
Public fogC1 As Single, fogC2 As Single, fogCol As Long
Public fogR As Single, fogG As Single, fogB As Single

Public Sub TexVTransform(vOut As tData, v As vData, M As Matrix)
Dim v1 As Vector, v2 As Vector, s As Single
  vOut.z = (v.x * M.m13 + v.y * M.m23 + v.z * M.m33 + M.m43)
  vOut.w = 1! / vOut.z
  vOut.x = (v.x * M.m11 + v.y * M.m21 + v.z * M.m31 + M.m41) * vOut.w * kXMul + kXAdd
  vOut.y = (v.x * M.m12 + v.y * M.m22 + v.z * M.m32 + M.m42) * vOut.w * kYMul + kYAdd
  
  If Draw <> DrawToZ Then
    vOut.u = v.u * TexS
    vOut.v = v.v * TexS
  
    VectorSubtract v1, vCam, Vector(v.x, v.y, v.z)
    VectorSubtract v2, vLite, Vector(v.x, v.y, v.z)
    s = lBrite / (v2.x * v2.x + v2.y * v2.y + v2.z * v2.z)
    VectorScale v2, v2, s
    vOut.cD = VectorDot(v.n, v2)
    VectorNormalize v1, v1
    VectorNormalize v2, v2
    VectorAdd v1, v1, v2
    VectorNormalize v1, v1
    vOut.cS = vOut.cD * lHalo2 / (lHalo1 - Abs(VectorDot(v.n, v1)))
  End If

  If Draw = DrawToZ Then
    vOut.Lz = vOut.z
    vOut.Lw = vOut.w
    vOut.Lu = vOut.x
    vOut.Lv = vOut.y
  End If
End Sub

Public Sub SetRTMain()
  kXAdd = SR_kXAdd
  kYAdd = SR_kYAdd
  kXMul = SR_kXMul
  kYMul = SR_kYMul
  RTWidth = SR_Width
  RTHeight = SR_Height
End Sub

Public Sub SetRTZ()
  kXAdd = 128
  kYAdd = 128
  kXMul = kXAdd * 3
  kYMul = kXMul
  RTWidth = 256
  RTHeight = 256
End Sub

Public Sub SetLightSz(v As Single)
  lHalo1 = 1 + 1 / v
  lHalo2 = 256 / v
End Sub

Public Sub SetLight(vEye As Vector, Angle As Single, Diff As Single, Brite As Single)
  Dim M1 As Matrix
  vLite = vEye
  lBrite = Brite
  MatrixTranslation MViewLight, -vEye.x, -vEye.y, -vEye.z
  MatrixRotationY M1, Angle
  MatrixMultiply MViewLight, MViewLight, M1
  MatrixRotationX M1, Diff
  MatrixMultiply MViewLight, MViewLight, M1
End Sub

Public Sub SetCamera(vEye As Vector, Angle As Single, Diff As Single)
  Dim M1 As Matrix
  vCam = vEye
  MatrixTranslation MView, -vEye.x, -vEye.y, -vEye.z
  MatrixRotationY M1, Angle
  MatrixMultiply MView, MView, M1
  MatrixRotationX M1, Diff
  MatrixMultiply MView, MView, M1
End Sub

Public Sub SR_SetTransformProj(ByVal fovY As Single, ByVal Aspect As Single)
  SR_kXAdd = SR_Width * 0.5
  SR_kYAdd = SR_Height * 0.5
  SR_kYMul = SR_kYAdd / Tan(fovY * 0.5)
  SR_kXMul = SR_kYMul * SR_Height * Aspect / SR_Width
End Sub

Public Function Color(ByVal a As Single, ByVal r As Single, ByVal g As Single, ByVal b As Single) As Color
  Color.a = a
  Color.r = r
  Color.g = g
  Color.b = b
End Function

Public Sub SR_Init(ByVal Width As Long, ByVal Height As Long, ByVal hDC As Long)
  SR_hDC = hDC
  SR_Width = Width
  SR_Height = Height
  With bi32BitInfo.bmiHeader
    .biBitCount = 32
    .biPlanes = 1
    .biSize = Len(bi32BitInfo.bmiHeader)
    .biWidth = SR_Width
    .biHeight = SR_Height
    .biSizeImage = SR_Width * SR_Height * 4
  End With
  ReDim CBuffer(SR_Width * SR_Height - 1) As Long
  ReDim WBuffer(SR_Width * SR_Height - 1) As Single
  ReDim ZBuffer(65535) As Single
End Sub

Public Sub SR_Present()
  SetDIBitsToDevice SR_hDC, 0&, 0&, SR_Width, SR_Height, 0&, 0&, 0&, SR_Height, CBuffer(0&), bi32BitInfo, 0&
End Sub

Public Sub SR_ClearCBuffer(ByVal c As Long)
  Dim x As Long, y As Long
  For x = 0& To SR_Width - 1&
    CBuffer(x) = c
  Next x
  x = SR_Width * 4&
  For y = SR_Width To SR_Width * (SR_Height - 1&) Step SR_Width
    CopyMemory CBuffer(y), CBuffer(y - SR_Width), x
  Next y
End Sub

Public Sub SR_ClearWBuffer(ByVal w As Single)
  Dim x As Long, y As Long
  For x = 0& To SR_Width - 1&
    WBuffer(x) = w
  Next x
  x = SR_Width * 4&
  For y = SR_Width To SR_Width * (SR_Height - 1&) Step SR_Width
    CopyMemory WBuffer(y), WBuffer(y - SR_Width), x
  Next y
End Sub

Public Sub SR_ClearZBuffer(ByVal z As Single)
  Dim x As Long, y As Long
  For x = 0& To 256& - 1&
    ZBuffer(x) = z
  Next x
  x = 256& * 4&
  For y = 256& To 256& * (256& - 1&) Step 256&
    CopyMemory ZBuffer(y), ZBuffer(y - 256&), x
  Next y
End Sub

